import os

from p3exporter.collector import CollectorConfig
from prometheus_client.core import GaugeMetricFamily

class LoadavgCollector(object):

    def __init__(self, config: CollectorConfig):
        """Instanciate a CpuCollector object."""
        pass

    def collect(self):
        """Collect load avg for 1, 5 and 15 minutes interval.

        Returns three gauge metrics. One for each load.
        """
        self.avg1, self.avg5, self.avg15 = os.getloadavg()
        yield GaugeMetricFamily('load_avg_1', "1m load average.", value=self.avg1)
        yield GaugeMetricFamily('load_avg_5', "5m load average.", value=self.avg5)
        yield GaugeMetricFamily('load_avg_15', "15m load average.", value=self.avg15)



"""Web module provide all parts to create the web app."""
from flask import Flask, render_template

from prometheus_client import make_wsgi_app
from werkzeug.middleware.dispatcher import DispatcherMiddleware

from p3exporter import CollectorConfig


def create_app(config: CollectorConfig):
    """Create the web app.

    This Function creates the flask app and dispatch metrics endpoint to prometheus wsgi app.

    :param config: A configuration object with data for template rendering.
    :type config: CollectorConfig
    :return: Created web app object.
    :rtype: DispatcherMiddleware
    """
    app = Flask(__name__, instance_relative_config=True)

    @app.route("/")
    def index(): # pylint: disable=W0612
        return render_template("index.html", title=config.exporter_name)

    dispatched_app = DispatcherMiddleware(app, {
        '/metrics': make_wsgi_app()
    })

    return dispatched_app



import random
import time

from p3exporter.collector import CollectorConfig
from prometheus_client.core import GaugeMetricFamily

class MyCollector(object):
    """A sample collector.

    It does not really do much. It only runs a method and return the time it runs as a gauge metric.
    """

    def __init__(self, config: CollectorConfig):
        """Instanciate a MyCollector object."""
        pass

    def collect(self):
        """Collect the metrics."""
        self.timer = time.perf_counter()
        _run_process()
        runtime = time.perf_counter() - self.timer
        yield GaugeMetricFamily('my_process_runtime', 'Time a process runs in seconds', value=runtime)

def _run_process():
    """Sample function to ran a command for metrics."""
    time.sleep(random.random()) # nosec



"""Entry point for collector sub module."""
import inflection
import logging

from importlib import import_module
from prometheus_client.core import REGISTRY


class CollectorConfig(object):
    """Class that provide all the logic needed for configuration handling."""

    def __init__(self, **kwargs):
        """Initialize instance variables.

        All configuration parameters are handed over as separate arguments.

        :raises Exception: Raises an exception if credentials are not well configured.
        """
        self.exporter_name = kwargs.pop('exporter_name', None)
        self.collectors = kwargs.pop('collectors', [])
        self.credentials = kwargs.pop('credentials', None)

        # do some fancy checks on configuration parameters
        if self.credentials is None:
            self.credentials = {}
        elif self.credentials is not None and (
                self.credentials['ssh_key'] is None or (
                    self.credentials['username'] is None or self.credentials['password'] is None)):
            raise Exception('Credential is not fully configured.')


class Collector(object):
    """Base class to load collectors.

    All collectors have to be placed inside the directory `collector`. You have to follow the naming convention:

    1. Place the collector code in a <name>.py file (e.g. `my.py`)
    2. Within the file <name>.py` a class <Name>Collector (e.g. `MyController`) needs to be defined.
       This is the main collector class which will be imported, instantiate and registered automatically.
    """

    def __init__(self, config: CollectorConfig):
        for c in config.collectors:
            try:
                collector_module = import_module("p3exporter.collector.{}".format(c), package=None)
                collector_class = getattr(collector_module, "{0}Collector".format(inflection.camelize(c)))
                collector = collector_class(config)
                REGISTRY.register(collector)
                logging.info("Collector '{0}' was loaded and registred sucessfully".format(c))
            except ModuleNotFoundError as e:
                logging.warning("Collector '{0}' not loaded: {1}".format(c, e.msg))
            except AttributeError as e:
                logging.warning("Collector '{0}' not loaded: {1}".format(c, e))



"""Init methods for p3exporter package."""
import argparse
from wsgiref.simple_server import make_server

import yaml
import logging
import signal
import sys
import time

from p3exporter.collector import Collector, CollectorConfig
from p3exporter.web import create_app


def shutdown():
    """Shutdown the app in a clean way."""
    logging.info('Shutting down, see you next time!')
    sys.exit(1)


def signal_handler(signum, frame):
    """Will be called if a signal was catched."""
    shutdown()


def main():
    """Start the application."""
    signal.signal(signal.SIGTERM, signal_handler)
    signal.signal(signal.SIGINT, signal_handler)

    logging.getLogger().setLevel(logging.INFO)

    parser = argparse.ArgumentParser(description="Python programmable Prometheus exporter.")
    parser.add_argument('-c', '--config', default='p3.yml',
                        help='path to configuration file.')
    parser.add_argument('-p', '--port', default=5876,
                        help='exporter exposed port')
    args = parser.parse_args()

    with open(args.config, 'r') as config_file:
        cfg = yaml.load(config_file, Loader=yaml.SafeLoader)
    collector_config = CollectorConfig(**cfg)

    Collector(collector_config)

    app = create_app(collector_config)

    logging.info("Start exporter, listen on {}".format(int(args.port)))
    httpd = make_server('', int(args.port), app)
    httpd.serve_forever()

    while True:
        time.sleep(5)



from setuptools import setup, find_packages

from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='p3exporter',
    version='1.0.0',
    description='Python Programmable Prometheus exporter',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://codeaffen.org/projects/p3exporter',
    author='Christian Meißner',
    author_email='cme@codeaffen.org',
    license='GPLv3',
    classifiers=[
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Topic :: System :: Monitoring',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
    keywords='monitoring prometheus exporter example',
    packages=find_packages(exclude=['tests']),
    include_package_data=True,
    zip_safe=False,
    package_data={'p3exporter': ['static/*', 'templates/*']},
    install_requires=[
        'inflection',
        'prometheus-client',
        'PyYAML',
        'flask',
    ],
    entry_points={
        'console_scripts': [
            'p3exporter=p3exporter:main',
        ],
    },
)
